/* Example plugin for snarkhunter by Brendan McKay.
   This only makes sense for girth 5.
   Reject all cubic graphs for which there is an edge not on a 5-cycle.
*/

#include "nausparse.h"
#include "snarkhunter.h"

#define FILTER filt
#define SUMMARY summ

static unsigned long long filt_total,filt_accepted;

int filt(GRAPH g, int n)
/* Returns 1 if there is an edge not lying on a cycle of length <= 5.
   Otherwise returns 0. */
{
    setword w,twostep[MAXN];
    int i;
    unsigned char *gi;

    ++filt_total;

    for (i = 0; i < n; ++i)
    {
        gi = g[i];
        twostep[i] =
            BIT(g[gi[0]][0]) | BIT(g[gi[0]][1]) | BIT(g[gi[0]][2]) | 
            BIT(g[gi[1]][0]) | BIT(g[gi[1]][1]) | BIT(g[gi[1]][2]) | 
            BIT(g[gi[2]][0]) | BIT(g[gi[2]][1]) | BIT(g[gi[2]][2]);
    }

    for (i = 0; i < n; ++i)
    {
        gi = g[i];
        w = twostep[i];
        if (gi[0] > i && POPC(w|twostep[gi[0]]) == 14) return 0;
        if (gi[1] > i && POPC(w|twostep[gi[1]]) == 14) return 0;
        if (gi[2] > i && POPC(w|twostep[gi[2]]) == 14) return 0;
    }

    ++filt_accepted;
    return 1;
}

void summ(void)
/* Write statistics collected by the filter. */
{
    fprintf(stderr,"Filter: %llu tested; %llu accepted\n",
                filt_total,filt_accepted);
}
